import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'package:zap_architecture/src/response.dart';
import 'package:zap_architecture/src/status.dart';

/// This handles the boilerplate for adding [Status] to a [ChangeNotifier].
/// Use that [ChangeNotifier] with [Provider] to make the [Status] data
/// available anywhere in your app.
///
/// For example:
///
/// 1. Wrap [MaterialApp] inside [MultiProvider] and then pass your
/// [ChangeNotifier] in [MultiProvider].
///
/// 2. Use [Consumer] widget (comes with 'provider' package) to get that
/// [ChangeNotifier].
///
/// 3. Access the [status] variable from that [ChangeNotifier] and then decide
/// if the app should show a loading indicator or an error page or show the
/// loaded data.
mixin StatusMixin on ChangeNotifier {
  Status _status = Status.clear();

  Status get status => _status;

  set status(Status taskStatus) {
    _status = taskStatus;
    notifyListeners();
  }
}

mixin ProcessMixin on ChangeNotifier {
  bool _isProcessing = false;

  bool get isProcessing => _isProcessing;

  set isProcessing(bool isProcessing) {
    _isProcessing = isProcessing;
    notifyListeners();
  }

  void startProcess() {
    assert(isProcessing == false,
        "ProcessMixin, can only handle one process at a time. You can't call 'startProcess' if it's already processing.");

    isProcessing = true;
  }

  void stopProcessing(Response response) {
    assert(isProcessing,
        "You called the 'stopProcessing' function when it was not processing or had finished processing, please go through your code again.");

    isProcessing = false;

    if (response.success == false) print("Showing error dialog!!!!");
  }
}

/// Provides booleans anywhere within a widget to check the operating system.
/// This helps you quickly develop platform-aware designs and widgets.
///
/// Example Usage:
///
/// 1. Add this to your Stateful/Stateless widget like so:
///   class MyStatelessPage extends StatelessWidget with PlatformsMixin {..}
///   OR
///   class MyStatelessPage extends StatefulWidget with PlatformsMixin {..}
///
/// 2. Use this anywhere inside that widget to access the boolean values:
///   [Stateless Widget]: 'isIOS' OR 'isAndroid'.
///   [Stateful Widget]: 'widget.isIOS' OR 'widget.isAndroid'.
///
/// 3. Use in IF statements to decide if to show Android UI or iOS UI. Example:
///       ....
///           Container(
///             child: widget.isIOS ? CupertinoButton(..) : RaisedButton(..)
///           )
///       ....
mixin PlatformsMixin on Widget {
  final bool isIOS = defaultTargetPlatform == TargetPlatform.iOS;
  final bool isAndroid = defaultTargetPlatform == TargetPlatform.android;
}

abstract class ErrorSafeBLOC {
  GestureTapCallback get onErrorTap;
}
